// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO24/DIO24_Linux_8.x.x.x_GSC_DN/driver/dio24.h $
// $Rev: 52088 $
// $Date: 2022-11-23 10:57:37 -0600 (Wed, 23 Nov 2022) $

// DIO24: Device Driver: header file

#ifndef	__DIO24_H__
#define	__DIO24_H__

#include "gsc_common.h"
#include "gsc_pci9080.h"



// macros *********************************************************************

#define	DIO24_BASE_NAME						"dio24"
#define	DIO24_BASE_NAME_LC					DIO24_BASE_NAME
#define	DIO24_BASE_NAME_UC					"DIO24"

// IOCTL command codes
#define	DIO24_IOCTL_REG_READ				OS_IOCTL_RW( 0, 12, gsc_reg_t)
#define	DIO24_IOCTL_REG_WRITE				OS_IOCTL_W ( 1, 12, gsc_reg_t)
#define	DIO24_IOCTL_REG_MOD					OS_IOCTL_W ( 2, 12, gsc_reg_t)
#define	DIO24_IOCTL_QUERY					OS_IOCTL_RW( 3,  4, s32)
#define	DIO24_IOCTL_INITIALIZE				OS_IOCTL   ( 4)
#define	DIO24_IOCTL_GPIO_DIR_OUT			OS_IOCTL_RW( 5,  4, s32)
#define	DIO24_IOCTL_GPIO_RX					OS_IOCTL_R ( 6,  4, s32)
#define	DIO24_IOCTL_GPIO_TX					OS_IOCTL_RW( 7,  4, s32)
#define	DIO24_IOCTL_IRQ_FALL_ENABLE			OS_IOCTL_RW( 8,  4, s32)
#define	DIO24_IOCTL_IRQ_RISE_ENABLE			OS_IOCTL_RW( 9,  4, s32)
#define	DIO24_IOCTL_WAIT_CANCEL				OS_IOCTL_RW(10, 28, dio24_wait_t)
#define	DIO24_IOCTL_WAIT_EVENT				OS_IOCTL_RW(11, 28, dio24_wait_t)
#define	DIO24_IOCTL_WAIT_STATUS				OS_IOCTL_RW(12, 28, dio24_wait_t)
#define	DIO24_IOCTL_CLOCK_COUNT_RX			OS_IOCTL_R (13,  4, s32)
#define	DIO24_IOCTL_CLOCK_LATCH_FALL_RX		OS_IOCTL_RW(14,  4, s32)
#define	DIO24_IOCTL_CLOCK_LATCH_RISE_RX		OS_IOCTL_RW(15,  4, s32)

// data types *****************************************************************

typedef struct
{
	u32	flags;		// done, timeout, cancel, ...
	u32	main;		// interrupts: PCI, DMA0, DMA1, Local, Other
	u32	rise;		// Individual low-to-high interrupt sources.
	u32	fall;		// Individual high-to-low interrupt sources.
	u32	reserved;	// RESERVED
	u32	timeout_ms;	// milliseconds
	u32	count;		// status: number of awaiting threads meeting any criteria
					// cancel: number of waits cancelled
} dio24_wait_t;



//*****************************************************************************
// DIO24_IOCTL_REG_READ
// DIO24_IOCTL_REG_WRITE
// DIO24_IOCTL_REG_MOD
//
#define	DIO24_REG_ENCODE(s,o)				GSC_REG_ENCODE(GSC_REG_TYPE_BAR2,(s),(o))
// Parameter:	gsc_reg_t*
#define	DIO24_GSC_FRR						DIO24_REG_ENCODE(4,0x000)// Firmware Revision Register
#define	DIO24_GSC_BCR						DIO24_REG_ENCODE(4,0x004)// Board Control Register
#define	DIO24_GSC_BSR						DIO24_REG_ENCODE(4,0x008)// Board Status Register
#define	DIO24_GSC_IRSR						DIO24_REG_ENCODE(4,0x020)// Interrupt Rising Status Register
#define	DIO24_GSC_IFSR						DIO24_REG_ENCODE(4,0x024)// Interrupt Falling Status Register
#define	DIO24_GSC_IRCR						DIO24_REG_ENCODE(4,0x028)// Interrupt Rising Control Register
#define	DIO24_GSC_IFCR						DIO24_REG_ENCODE(4,0x02C)// Interrupt Falling Control Register
#define	DIO24_GSC_DDIILR					DIO24_REG_ENCODE(4,0x030)// Discrete Data Input IRQ Latch Register
#define	DIO24_GSC_CCR						DIO24_REG_ENCODE(4,0x034)// Clock COunter Register
#define	DIO24_GSC_IOCR						DIO24_REG_ENCODE(4,0x060)// I/O Control Register
#define	DIO24_GSC_DDOR						DIO24_REG_ENCODE(4,0x064)// Discrete Data Output Register
#define	DIO24_GSC_DDIR						DIO24_REG_ENCODE(4,0x068)// Discrete Data Input Register
#define	DIO24_GSC_CCLRI0R					DIO24_REG_ENCODE(4,0x100)// Clock Counter Latch Rising Interrupt 0 Register
#define	DIO24_GSC_CCLRI1R					DIO24_REG_ENCODE(4,0x104)// Clock Counter Latch Rising Interrupt 1 Register
#define	DIO24_GSC_CCLRI2R					DIO24_REG_ENCODE(4,0x108)// Clock Counter Latch Rising Interrupt 2 Register
#define	DIO24_GSC_CCLRI3R					DIO24_REG_ENCODE(4,0x10C)// Clock Counter Latch Rising Interrupt 3 Register
#define	DIO24_GSC_CCLFI0R					DIO24_REG_ENCODE(4,0x160)// Clock Counter Latch Falling Interrupt 0 Register
#define	DIO24_GSC_CCLFI1R					DIO24_REG_ENCODE(4,0x164)// Clock Counter Latch Falling Interrupt 1 Register
#define	DIO24_GSC_CCLFI2R					DIO24_REG_ENCODE(4,0x168)// Clock Counter Latch Falling Interrupt 2 Register
#define	DIO24_GSC_CCLFI3R					DIO24_REG_ENCODE(4,0x16C)// Clock Counter Latch Falling Interrupt 3 Register

//*****************************************************************************
// DIO24_IOCTL_QUERY
//
//	Parameter:	s32
//		Pass in a value from the list below.
//		The value returned is the answer to the query.

typedef enum
{
		DIO24_QUERY_COUNT,				// Number of query options.
		DIO24_QUERY_DEVICE_TYPE,		// Value from gsc_dev_type_t
		DIO24_QUERY_JUMPER_QTY,			// How many jumpers are there?
		DIO24_QUERY_JUMPER_SENSE,		// Jumper is on if it is this value.
		DIO24_QUERY_JUMPER_VAL,			// user jumpers, BSR D16-D17
		DIO24_QUERY_DI,					// Does the device support the Dedicated Input?
		DIO24_QUERY_IRQ_QTY,			// Number of firmware supported interrupts.
		DIO24_QUERY_CLOCK_COUNTER,		// Is the Clock Counter feature present?
		DIO24_QUERY_LAST
} dio24_query_t;

#define	DIO24_IOCTL_QUERY_ERROR			(-1)

//*****************************************************************************
// DIO24_IOCTL_INITIALIZE				BCR D0
//
//	Parameter:	unused

//*****************************************************************************
// DIO24_IOCTL_GPIO_DIR_OUT				IOCR D0-D9
//
//	Parameter:	s32
//		Pass in an bitwise oring of the below listed options,
//		or -1 to read the current setting.
#define	DIO24_GPIO_PORT_A				0x001
#define	DIO24_GPIO_PORT_B				0x002
#define	DIO24_GPIO_PORT_C0				0x004
#define	DIO24_GPIO_PORT_C1				0x008
#define	DIO24_GPIO_PORT_C2				0x010
#define	DIO24_GPIO_PORT_C3				0x020
#define	DIO24_GPIO_PORT_C4				0x040
#define	DIO24_GPIO_PORT_C5				0x080
#define	DIO24_GPIO_PORT_C6				0x100
#define	DIO24_GPIO_PORT_C7				0x200
#define	DIO24_GPIO_PORT_C				0x3FC
#define	DIO24_GPIO_PORT_ABC				0x3FF

//*****************************************************************************
// DIO24_IOCTL_GPIO_RX					DDIR D0-D24
//
//	Parameter:	s32
//		Valid values returned are from zero to 0x1FFFFFF.

//*****************************************************************************
// DIO24_IOCTL_GPIO_TX					DDOR D0-D23
//
//	Parameter:	s32
//		Pass in a value from zero to 0xFFFFFF,
//		or -1 to read the current setting.

//*****************************************************************************
// DIO24_IOCTL_IRQ_FALL_ENABLE			IFCR D0-D23
// DIO24_IOCTL_IRQ_RISE_ENABLE			IRCR D0-D23
//
//	Parameter:	s32
//		Pass in a value from zero to 0xFFFFFF,
//		or -1 to read the current setting.
//		The value -1 is returned if the feature is unsupported.

//*****************************************************************************
// DIO24_IOCTL_WAIT_EVENT		all fields must be valid
// DIO24_IOCTL_WAIT_CANCEL		fields need not be valid
// DIO24_IOCTL_WAIT_STATUS		fields need not be valid
//
//	Parameter:	dio24_wait_t*
// dio24_wait_t.flags - see gsc_wait_t.flags in gsc_common.h
// dio24_wait_t.main - see gsc_wait_t.main in gsc_common.h
// dio24_wait_t.rise - Valid values are from zero to below ..._ALL macro
//						which represent each of the 24 RISE interrupts.
#define	DIO24_WAIT_RISE_ALL				0xFFFFFF	// IRCR D0-D23
// dio24_wait_t.fall - Valid values are from zero to below ..._ALL macro
//						which represent each of the 24 FALL interrupts.
#define	DIO24_WAIT_FALL_ALL				0xFFFFFF	// IFCR D0-D23
// dio24_wait_t.reserved - <ust always be zero.
// dio24_wait_t.timeout_ms - see gsc_wait_t.timeout_ms in gsc_common.h
// dio24_wait_t.count - see gsc_wait_t.count in gsc_common.h

//*****************************************************************************
// DIO24_IOCTL_CLOCK_COUNT_RX			CCR D0-D31
//
//	Parameter:	u32
//		Valid values returned are from zero to 0xFFFFFFFF.

//*****************************************************************************
// DIO24_IOCTL_CLOCK_LATCH_FALL_RX		CCFIXLR D0-D31
// DIO24_IOCTL_CLOCK_LATCH_RISE_RX		CCRIXLR D0-D31
//
//	Parameter:	u32
//		Pass in values from zero to three.
//		Valid values returned are from zero to 0xFFFFFFFF.



#endif
