// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO24/DIO24_Linux_8.x.x.x_GSC_DN/samples/b2btest/main.c $
// $Rev: 52993 $
// $Date: 2023-05-03 16:47:09 -0500 (Wed, 03 May 2023) $

// DIO24: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _parse_args(int argc, char** argv, args_t* args)
{
	const gsc_arg_item_t	list[]	=
	{
		//	type				var						arg			values			desc
		{	GSC_ARG_S32_FLAG,	&args->continuous,		"-c",		{ -1 },			"Continue testing until an error occurs."							},
		{	GSC_ARG_S32_FLAG,	&args->continuous,		"-C",		{ +1 },			"Continue testing even if errors occur."							},
		{	GSC_ARG_S32_MIN,	&args->minute_limit,	"-m",		{ 1 },			"Continue testing for up to # minutes."								},
		{	GSC_ARG_S32_RANGE,	&args->ms_wait,			"-ms",		{ 1, 1000 },	"Wait for # ms before each input read."								},
		{	GSC_ARG_S32_MIN,	&args->test_limit,		"-n",		{ 1 },			"Continue testing for up to # iterations."							},
		{	GSC_ARG_S32_FLAG,	&args->pause,			"-p",		{ 1 },			"Pause after each individual bit test (adds -v)."					},
		{	GSC_ARG_S32_RANGE,	&args->rx_qty,			"-rx",		{ 0, 1000 },	"Read the input # times for validation (0 disables, default: 1)."	},
		{	GSC_ARG_S32_FLAG,	&args->verbose,			"-v",		{ 1 },			"Work verbosely?"													},
		{	GSC_ARG_DEV_INDEX,	&args->index[0],		"index_1",	{ 1 },			"The zero based index of the first device to access."				},
		{	GSC_ARG_DEV_INDEX,	&args->index[1],		"index_2",	{ 2 },			"The zero based index of the second device to access."				}
	};

	const gsc_arg_set_t	set	=
	{
		/* name	*/	"b2btest",
		/* desc	*/	"Perform board-to-board testing.",
		/* qty	*/	SIZEOF_ARRAY(list),
		/* list	*/	list
	};

	int	errs;

	memset(args, 0, sizeof(args_t));
	args->rx_qty	= 1;
	args->index[0]	= 0;
	args->index[1]	= 1;

	errs	= gsc_args_parse(argc, argv, &set);

	if (args->pause)
		args->verbose	= 1;

	return(errs);
}



//*****************************************************************************
static void _show_access_index(const args_t* args)
{
	gsc_label("Accessing Devices");
	printf("%d and %d\n", args->index[0], args->index[1]);
}



//*****************************************************************************
static void _show_time_stamp(void)
{
	const char*	psz;
	struct tm*	stm;
	time_t		tt;

	time(&tt);
	stm	= localtime(&tt);
	psz	= (char*) asctime(stm);
	gsc_label("Time Stamp");
	printf("%s", psz);
}



/******************************************************************************
*
*	Function:	main
*
*	Purpose:
*
*		Control the overall flow of the application.
*
*	Arguments:
*
*		argc			The number of command line arguments.
*
*		argv			The list of command line arguments.
*
*	Returned:
*
*		EXIT_SUCCESS	No errors were encounterred.
*		EXIT_FAILURE	One or more problems were encounterred.
*
******************************************************************************/

int main(int argc, char** argv)
{
	args_t	args;
	int		errs;
	time_t	exec		= time(NULL);
	long	failures	= 0;
	long	hours;
	long	mins;
	time_t	now;
	long	passes		= 0;
	int		ret;
	long	secs;
	time_t	t_limit;
	time_t	test;
	long	tests		= 0;

	for (;;)	// A convenience loop.
	{
		gsc_label_init(25);
		test	= time(NULL);
		errs	= _parse_args(argc, argv, &args);

		// Introduction

		t_limit	= exec + (args.minute_limit * 60);
		os_id_host();
		_show_time_stamp();
		_show_access_index(&args);
		errs	+= dio24_init_util(1);
		errs	+= os_id_driver(dio24_open, dio24_read, dio24_close);
		errs	+= dio24_count_boards(1, &args.qty);

		if ((errs == 0) && (args.qty < 2))
		{
			errs++;
			printf("FAIL <---  (Too few devices present.)\n");
		}

		// Open access to device

		if (errs == 0)
		{
			errs	+= dio24_open_util(args.index[0], 0, args.index[0], 1, &args.fd[0]);
			errs	+= dio24_open_util(args.index[1], 0, args.index[1], 1, &args.fd[1]);
		}

		if (errs)
			args.continuous	= 0;

		if (errs == 0)
		{
			// Device identification

			errs	+= dio24_id_device(args.fd[0], args.index[0], 1);
			errs	+= dio24_id_device(args.fd[1], args.index[1], 1);

			// Perform testing

			errs	+= perform_tests(&args);
		}

		// Close device access

		errs	+= dio24_close_util(args.fd[0], args.index[0], 1);
		errs	+= dio24_close_util(args.fd[1], args.index[1], 1);

		// End of processing.

		now	= time(NULL);
		tests++;

		if (errs)
		{
			failures++;
			printf(	"\nRESULTS: FAIL <---  (%d error%s)",
					errs,
					(errs == 1) ? "" : "s");
		}
		else
		{
			passes++;
			printf("\nRESULTS: PASS");
		}

		secs	= (long) (now - test);
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(" (duration %ld:%ld:%02ld)\n", hours, mins, secs);

		secs	= (long) (now - exec);
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(	"SUMMARY: tests %ld, pass %ld, fail %ld"
				" (duration %ld:%ld:%02ld)\n\n",
				tests,
				passes,
				failures,
				hours,
				mins,
				secs);

		if (args.continuous == 0)
			break;

		if ((args.continuous < 0) && (errs))
			break;

		if ((args.minute_limit > 0) && (now >= t_limit))
			break;

		if ((args.test_limit > 0) && (tests >= args.test_limit))
			break;
	}

	ret	= ((args.qty <= 0) || (errs) || (failures)) ? EXIT_FAILURE : EXIT_SUCCESS;
	return(ret);
}


