// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO24/DIO24_Linux_8.x.x.x_GSC_DN/samples/b2btest/perform.c $
// $Rev: 52242 $
// $Date: 2022-12-15 09:09:45 -0600 (Thu, 15 Dec 2022) $

// DIO24: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _config_rx(const args_t* args, int index)
{
	char	buf[64];
	int		errs	= 0;

	sprintf(buf, "Input - #%d", (int) args->index[index]);
	gsc_label(buf);
	printf("\n");
	gsc_label_level_inc();

	errs	+= dio24_initialize		(args->fd[index], -1, 1);
	errs	+= dio24_gpio_dir_out	(args->fd[index], -1, 1, 0, NULL);

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
static int _config_tx(const args_t* args, int index)
{
	char	buf[64];
	int		errs	= 0;

	sprintf(buf, "Output - #%d", (int) args->index[index]);
	gsc_label(buf);
	printf("\n");
	gsc_label_level_inc();

	errs	+= dio24_initialize		(args->fd[index], -1, 1);
	errs	+= dio24_gpio_dir_out	(args->fd[index], -1, 1, DIO24_GPIO_PORT_ABC, NULL);

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
static void _pause_befor_next(const args_t* args)
{
	if (args->pause)
	{
		printf("  Hit return to continue.");
		getchar();
		printf("\n");
	}
}



//*****************************************************************************
static int _walking_bit(const args_t* args, int tx, int rx, int one)
{
	char	buf[64];
	int		err;
	int		errs	= 0;
	int		pos;
	int		rx_qty;
	s32		val_in;
	s32		val_out;

	sprintf(buf, "Walking %s", one ? "1" : "0");
	gsc_label(buf);

	if (args->verbose)
	{
		printf("\n");
		gsc_label_level_inc();
	}

	for (pos = 0; pos <= 23; pos++)
	{
		val_out	= 0x1 << pos;

		if (one != 1)
			val_out	= ~val_out & 0xFFFFFF;

		if (args->verbose)
		{
			sprintf(buf, "0x%06lX", (long) val_out);
			gsc_label(buf);
		}

		err		= dio24_gpio_tx(args->fd[tx], -1, 0, val_out, &val_in);
		val_in	&= 0xFFFFFF;

		if (err)
		{

			if (args->verbose)
				printf("FAIL <---  (GPIO output request failed.)\n");

			errs++;

			_pause_befor_next(args);
			continue;
		}

		if (val_out != val_in)
		{
			if (args->verbose)
				printf("FAIL <---  (GPIO output readback differs: 0x%06lX)\n", (long) val_in);

			errs++;
			_pause_befor_next(args);
			continue;
		}

		for (rx_qty = 0; rx_qty < args->rx_qty; rx_qty++)
		{
			if (args->ms_wait)
				os_sleep_ms(args->ms_wait);

			err		= dio24_gpio_rx(args->fd[rx], -1, 0, &val_in);
			val_in	&= 0xFFFFFF;

			if (err)
			{
				if (args->verbose)
					printf("FAIL <---  (GPIO input request failed.)\n");

				errs++;
				break;
			}

			if (val_out != val_in)
			{
				if (args->verbose)
					printf("FAIL <---  (GPIO input readback differs: 0x%06lX)\n", (long) val_in);

				errs++;
				break;
			}
		}

		if (errs)
		{
			_pause_befor_next(args);
			continue;
		}

		if (args->verbose)
			printf("PASS  (GPIO Input: 0x%06lX)\n", (long) val_in);

		_pause_befor_next(args);
	}

	if (args->verbose == 0)
		printf("%s\n", errs ? "FAIL <---" : "PASS");
	else
		gsc_label_level_dec();

	return(errs);
}



//*****************************************************************************
static int _b2b_test(const args_t* args, int tx, int rx)
{
	char	buf[64];
	int		errs	= 0;

	sprintf(buf, "GPIO #%d -> #%d", (int) args->index[tx], (int) args->index[rx]);
	gsc_label(buf);
	printf("\n");
	gsc_label_level_inc();

	errs	+= _config_tx(args, tx);
	errs	+= _config_rx(args, rx);
	errs	+= _walking_bit(args, tx, rx, 1);
	errs	+= _walking_bit(args, tx, rx, 0);

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int	errs	= 0;

	errs	+= _b2b_test(args, 0, 1);
	errs	+= _b2b_test(args, 1, 0);

	return(errs);
}


