// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO24/DIO24_Linux_8.x.x.x_GSC_DN/samples/irq/clock_counter.c $
// $Rev: 51515 $
// $Date: 2022-10-03 15:43:53 -0500 (Mon, 03 Oct 2022) $

// DIO24: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _pattern_test(int fd, int index, u32 rise, u32 fall, u32 bit)
{
	char	buf[32];
	u32		clock_1;
	u32		clock_2;
	int		errs		= 0;
	u32		latch_1;
	u32		latch_2;
	int		rollover	= 0;
	u32		val;

	sprintf(buf, "0x%06lX", (long) bit);
	gsc_label(buf);

	for (;; rollover++)
	{
		if (rollover > 2)
		{
			errs	= 1;
			printf("FAIL <---  (Two rollovers in two tests)\n");
			break;
		}

		errs	+= dio24_initialize		(fd, -1, 0);
		errs	+= dio24_gpio_dir_out	(fd, -1, 0, DIO24_GPIO_PORT_ABC, NULL);
		errs	+= dio24_irq_rise_enable(fd, -1, 0, 0, NULL);
		errs	+= dio24_irq_fall_enable(fd, -1, 0, 0, NULL);

		// Preset output value.
		val		= rise ? 0 : 0xFFFFFF;
		errs	+= dio24_gpio_tx(fd, -1, 0, val, NULL);

		// Enable interrupts
		errs	+= dio24_irq_rise_enable(fd, -1, 0, rise, NULL);
		errs	+= dio24_irq_fall_enable(fd, -1, 0, fall, NULL);

		errs	+= dio24_clock_count_rx(fd, -1, 0, &clock_1);

		if (rise)
			errs	+= dio24_clock_latch_rise_rx(fd, -1, 0, index, &latch_1);
		else
			errs	+= dio24_clock_latch_fall_rx(fd, -1, 0, index, &latch_1);

		gsc_time_sleep_ms(1);

		// Induce interrupt
		val		= rise ? rise : (0xFFFFFF & ~fall);
		errs	+= dio24_gpio_tx(fd, -1, 0, val, NULL);

		// Restore outputs
		val		= rise ? 0 : 0xFFFFFF;
		errs	+= dio24_gpio_tx(fd, -1, 0, val, NULL);

		if (rise)
			errs	+= dio24_clock_latch_rise_rx(fd, -1, 0, index, &latch_2);
		else
			errs	+= dio24_clock_latch_fall_rx(fd, -1, 0, index, &latch_2);

		gsc_time_sleep_ms(1);
		errs	+= dio24_clock_count_rx(fd, -1, 0, &clock_2);

		if (errs)
		{
			printf("FAIL <---  (%d error%s reported)\n", errs, (errs == 1) ? "" : "s");
			errs	= 1;
			break;
		}

		if (clock_1 > clock_2)	// There may have been a rollover.
			continue;

		if (latch_2 == latch_1)
		{
			printf("FAIL <---  (Latched value did not change.)\n");
			errs	= 1;
			break;
		}

		if (latch_2 <= clock_1)
		{
			printf("FAIL <---  (Latched value is not greater than pre-IRQ clock.)\n");
			errs	= 1;
			break;
		}

		if (latch_2 >= clock_2)
		{
			printf("FAIL <---  (Latched value is not less than post-IRQ clock.)\n");
			errs	= 1;
			break;
		}

		printf("PASS\n");
		break;
	}

	return(errs);
}



//*****************************************************************************
static int _counter_test(int fd)
{
	u32	count;
	int	down		= 0;
	int	errs;
	int	i;
	u32	last;
	int	same		= 0;

	gsc_label("Clock Counter");

	errs	= dio24_clock_count_rx(fd, -1, 0, &last);

	for (i = 0; i < 1000; i++)
	{
		gsc_time_sleep_ms(1);
		errs	+= dio24_clock_count_rx(fd, -1, 0, &count);

		if (count == last)
			same++;

		if (count < last)
			down++;

		last	= count;
	}

	if (errs)
	{
		errs	= 1;
		printf("FAIL  (count read failures)\n");
	}
	else if (same)
	{
		errs	= 1;
		printf(	"FAIL  (count unchanged %d time%s)\n",
				same,
				(same == 1) ? "" : "s");
	}
	else if (down > 1)
	{
		errs	= 1;
		printf("FAIL  (count decreased %d times)\n", down);
	}
	else
	{
		printf(	"PASS  (%d reads, %d decrease%s)\n",
				i,
				down,
				(down == 1) ? "" : "s");
	}

	return(errs);
}



//*****************************************************************************
static int _rising_latch_tests(int fd)
{
	int	errs	= 0;

	gsc_label("Rising Edge Tests");
	printf("\n");
	gsc_label_level_inc();

	errs	+= _pattern_test(fd, 0, 0x01, 0, 0x01);
	errs	+= _pattern_test(fd, 1, 0x02, 0, 0x02);
	errs	+= _pattern_test(fd, 2, 0x04, 0, 0x04);
	errs	+= _pattern_test(fd, 3, 0x08, 0, 0x08);

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
static int _falling_latch_tests(int fd)
{
	int	errs	= 0;

	gsc_label("Falling Edge Tests");
	printf("\n");
	gsc_label_level_inc();

	errs	+= _pattern_test(fd, 0, 0, 0x01, 0x01);
	errs	+= _pattern_test(fd, 1, 0, 0x02, 0x02);
	errs	+= _pattern_test(fd, 2, 0, 0x04, 0x04);
	errs	+= _pattern_test(fd, 3, 0, 0x08, 0x08);

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
static int _clock_latch_test(int fd, s32 irq)
{
	int	errs	= 0;

	gsc_label("Clock Latch");

	if (irq == 0)
	{
		printf("SKIPING  (Interrupts not supported by firmware.)\n");
	}
	else
	{
		printf("\n");
		gsc_label_level_inc();

		errs	+= _rising_latch_tests(fd);
		errs	+= _falling_latch_tests(fd);

		gsc_label_level_dec();
	}

	return(errs);
}



//*****************************************************************************
int clock_counter_tests(int fd)
{
	s32	clock;
	int	errs	= 0;
	s32	irq;

	errs	+= dio24_query(fd, -1, 0, DIO24_QUERY_CLOCK_COUNTER, &clock);
	errs	+= dio24_query(fd, -1, 0, DIO24_QUERY_IRQ_QTY, &irq);
	gsc_label("Clock Counter Tests");

	if (errs)
	{
		printf("ABORTING DUE TO ERROR.\n");
	}
	else if (clock == 0)
	{
		printf("SKIPING  (Clock counter not supported by firmware.)\n");
	}
	else
	{
		printf("\n");
		gsc_label_level_inc();

		errs	+= _counter_test(fd);
		errs	+= _clock_latch_test(fd, irq);

		gsc_label_level_dec();
	}

	return(errs);
}


