// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO24/DIO24_Linux_8.x.x.x_GSC_DN/samples/irq/input_latch.c $
// $Rev: 49894 $
// $Date: 2022-01-12 12:00:23 -0600 (Wed, 12 Jan 2022) $

// DIO24: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _pattern_test(int fd, int index, u32 rise, u32 fall, u32 bit)
{
	char	buf[32];
	int		errs	= 0;
	u32		expect;
	u32		got;
	u32		pattern	= (0xAAAAAAAA >> (index & 0x1)) & 0xFFFFFF;

	sprintf(buf, "0x%06lX", (long) bit);
	gsc_label(buf);

	errs	+= dio24_gpio_dir_out	(fd, -1, 0, DIO24_GPIO_PORT_ABC, NULL);
	errs	+= dio24_irq_rise_enable(fd, -1, 0, 0, NULL);
	errs	+= dio24_irq_fall_enable(fd, -1, 0, 0, NULL);

	if (rise)
	{
		errs	+= dio24_gpio_tx(fd, -1, 0, 0xFFFFFF & (pattern & ~rise), NULL);
		expect	= pattern | rise;
	}
	else
	{
		errs	+= dio24_gpio_tx(fd, -1, 0, 0xFFFFFF & (pattern | fall), NULL);
		expect	= pattern & ~fall;
	}

	errs	+= dio24_irq_rise_enable(fd, -1, 0, rise, NULL);
	errs	+= dio24_irq_fall_enable(fd, -1, 0, fall, NULL);

	if (rise)
	{
		errs	+= dio24_gpio_tx(fd, -1, 0, 0xFFFFFF & (pattern | rise), NULL);
		errs	+= dio24_gpio_tx(fd, -1, 0, 0xFFFFFF & ~(pattern | rise), NULL);
	}
	else
	{
		errs	+= dio24_gpio_tx(fd, -1, 0, 0xFFFFFF & pattern & ~fall, NULL);
		errs	+= dio24_gpio_tx(fd, -1, 0, 0xFFFFFF & ~(pattern & ~fall), NULL);
	}

	errs	+= dio24_reg_read(fd, -1, 0, DIO24_GSC_DDIILR, &got);

	if (errs)
	{
		printf("FAIL <---  (%d error%s reported)\n", errs, (errs == 1) ? "" : "s");
		errs	= 1;
	}
	else if (got != expect)
	{
		errs++;
		printf("FAIL <---  (expected 0x%06lX, got 0x%06lX)\n", (long) expect, (long) got);
	}
	else
	{
		printf("PASS\n");
	}

	return(errs);
}



//*****************************************************************************
static int _rising_latch_tests(int fd)
{
	u32	bit;
	int	errs	= 0;
	int	i;

	gsc_label("Rising Edge Tests");
	printf("\n");
	gsc_label_level_inc();

	for (i = 0; i < 24; i++)
	{
		bit		= 0x1 << i;
		errs	+= _pattern_test(fd, i, bit, 0, bit);
	}

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
static int _falling_latch_tests(int fd)
{
	u32	bit;
	int	errs	= 0;
	int	i;

	gsc_label("Falling Edge Tests");
	printf("\n");
	gsc_label_level_inc();

	for (i = 0; i < 24; i++)
	{
		bit		= 0x1 << i;
		errs	+= _pattern_test(fd, i, 0, bit, bit);
	}

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
int input_latch_tests(int fd)
{
	int	errs;
	s32	irq;

	errs	= dio24_query(fd, -1, 0, DIO24_QUERY_IRQ_QTY, &irq);
	gsc_label("Input Latch Tests");

	if (errs)
	{
		printf("ABORTING DUE TO ERROR.\n");
	}
	else if (irq <= 0)
	{
		printf("SKIPING  (Interrupts not supported by firmware.)\n");
	}
	else
	{
		printf("\n");
		gsc_label_level_inc();

		errs	+= _rising_latch_tests(fd);
		errs	+= _falling_latch_tests(fd);

		gsc_label_level_dec();
	}

	return(errs);
}


