// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO24/DIO24_Linux_8.x.x.x_GSC_DN/samples/lbtest/perform.c $
// $Rev: 51516 $
// $Date: 2022-10-03 15:44:05 -0500 (Mon, 03 Oct 2022) $

// DIO24: Sample Application: source file

#include "main.h"




// data types *****************************************************************

typedef struct
{
	const char*	name;
	s32			dir_out;
	int			offset;		// of base output bit: 0, 8, or 16
} data_t;



// variables ******************************************************************

static const data_t	list[]	=
{
	// name					dir_out				offset
	{ "Port A Loopback",	DIO24_GPIO_PORT_A,	0	},
	{ "Port B Loopback",	DIO24_GPIO_PORT_B,	8	},
	{ "Port C Loopback",	DIO24_GPIO_PORT_C,	16	},
	{ NULL }
};



//*****************************************************************************
static int _output_test(int fd, int offset, int verbose)
{
	int	cr		= 0;
	int	errs	= 0;
	int	i;
	s32	rx;
	u32	rx_exp;		// expected rx value;
	s32	tx;

	gsc_label("Time Stamp");

	for (i = 0; i <= 255; i++)
	{
		tx		= i << offset;
		errs	+= dio24_gpio_tx(fd, -1, 0, tx, NULL);
		rx_exp	= (i <<  0)
				| (i <<  8)
				| (i << 16)
				| (i << 24 & 0x1000000);

		os_sleep_ms(5);
		errs	+= dio24_gpio_rx(fd, -1, 0, &rx);

		if (rx != rx_exp)
		{
			if ((cr == 0) && (verbose))
			{
				printf("\n");
				cr	= 1;
			}

			errs++;

			if (verbose == 0)
				continue;

			printf("ERROR: % 3d. Tx 0x%08lX, Rx 0x%08lX, Rx Exp 0x%08lX\n", i, (long) tx, (long) rx, (long) rx_exp);
		}
	}

	printf("%s  (input A/B/C/DI)\n", errs ? "FAIL <---" : "PASS");
	return(errs);
}



//*****************************************************************************
static int _lb_test(int fd, int index, int verbose)
{
	int	errs	= 0;

	gsc_label(list[index].name);
	printf("\n");
	gsc_label_level_inc();

	errs	+= dio24_initialize		(fd, -1, 1);
	errs	+= dio24_gpio_dir_out	(fd, -1, 1, list[index].dir_out, NULL);
	errs	+= _output_test			(fd, list[index].offset, verbose);

	gsc_label_level_dec();
	return(errs);
}


//*****************************************************************************
int perform_tests(const args_t* args)
{
	int	errs	= 0;
	int	i;

	for (i = 0; list[i].name; i++)
		errs	+= _lb_test(args->fd, i, args->verbose);

	return(errs);
}


