// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO24/DIO24_Linux_8.x.x.x_GSC_DN/utils/util_id.c $
// $Rev: 51511 $
// $Date: 2022-10-03 15:40:43 -0500 (Mon, 03 Oct 2022) $

// DIO24: Utilities: source file

#include "main.h"



//*****************************************************************************
static int _id_device_pci(int fd)
{
	int	errs	= 0;
	u32	reg;

	gsc_label("Vendor ID");
	errs	+= dio24_reg_read(fd, -1, 0, GSC_PCI_9080_VIDR, &reg);
	printf("0x%04lX      ", (long) reg);

	if (reg == 0x10B5)
	{
		printf("(PLX)\n");
	}
	else
	{
		errs++;
		printf("(UNKNOWN) FAIL <---\n");
	}

	gsc_label("Device ID");
	errs	+= dio24_reg_read(fd, -1, 0, GSC_PCI_9080_DIDR, &reg);
	printf("0x%04lX      ", (long) reg);

	if (reg == 0x9080)
	{
		printf("(PCI9080)\n");
	}
	else
	{
		errs++;
		printf("(UNKNOWN) FAIL <---\n");
	}

	gsc_label("Sub Vendor ID");
	errs	+= dio24_reg_read(fd, -1, 0, GSC_PCI_9080_SVID, &reg);
	printf("0x%04lX      ", (long) reg);

	if (reg == 0x10B5)
	{
		printf("(PLX)\n");
	}
	else
	{
		errs++;
		printf("(UNKNOWN) FAIL <---\n");
	}

	gsc_label("Subsystem ID");
	errs	+= dio24_reg_read(fd, -1, 0, GSC_PCI_9080_SID, &reg);
	printf("0x%04lX      ", (long) reg);

	if (reg == 0x2706)
	{
		printf("(DIO24)\n");
	}
	else
	{
		errs++;
		printf("(UNKNOWN) FAIL <---\n");
	}

	return(errs);
}



//*****************************************************************************
static int _id_device_frr(int fd)
{
	int			errs;
	const char*	name;
	u32			val;

	name	= dio24_reg_get_name(DIO24_GSC_FRR);
	gsc_label(name);
	errs	= dio24_reg_read(fd, -1, 0, DIO24_GSC_FRR, &val);

	if (errs)
		printf("FAIL <---\n");
	else
		printf("0x%08lX\n", (long) val);

	return(errs);
}



//*****************************************************************************
static int _id_user_jumpers(int fd)
{
	u32			bit;
	const char*	comma	= "";
	int			errs	= 0;
	int			i;
	s32			qty;
	s32			sense;
	s32			val;

	gsc_label("User Jumpers");
	errs	+= dio24_query(fd, -1, 0, DIO24_QUERY_JUMPER_VAL, &val);
	errs	+= dio24_query(fd, -1, 0, DIO24_QUERY_JUMPER_QTY, &qty);
	errs	+= dio24_query(fd, -1, 0, DIO24_QUERY_JUMPER_SENSE, &sense);

	if (errs)
		printf("FAIL <---  (");

	printf("0x%lX:", (long) val);

	for (i = qty; --i >= 0;)
	{
		bit	= 0x1 << i;

		if (val & bit)
			printf("%s %s", comma, sense ? "On" : "Off");
		else
			printf("%s %s", comma, sense ? "Off" : "On");

		comma	= ",";
	}

	if (errs)
		printf(")");

	printf("\n");
	return(errs);
}



//*****************************************************************************
int dio24_id_device(int fd, int index, int verbose)
{
	s32			device_type;
	int			errs;
	const char*	psz		= "";

	if (verbose)
		gsc_label_index("Device", index);

	errs	= dio24_query(fd, -1, 0, DIO24_QUERY_DEVICE_TYPE, &device_type);

	if (errs)
	{
		if (verbose)
			printf("FAIL <---  (query error)\n");
	}
	else
	{
		switch (device_type)
		{
			default:

				errs	= 1;

				if (verbose)
				{
					printf(	"FAIL <---  (unexpected device type: %ld)",
							(long) device_type);
				}

				break;

			case GSC_DEV_TYPE_DIO24:	psz	= "DIO24";	break;
		}

		if ((verbose) && (errs == 0))
			printf("%s\n", psz);

		gsc_label_level_inc();

		errs	+= _id_device_pci(fd);
		errs	+= _id_device_frr(fd);
		errs	+= _id_user_jumpers(fd);

		gsc_label_level_dec();
	}

	return(errs);
}



//*****************************************************************************
int dio24_id_driver(void)
{
	int	errs;

	errs	= os_id_driver(dio24_open, dio24_read, dio24_close);
	return(errs);
}


