// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO24/DIO24_Linux_8.x.x.x_GSC_DN/utils/util_query.c $
// $Rev: 51511 $
// $Date: 2022-10-03 15:40:43 -0500 (Mon, 03 Oct 2022) $

// DIO24: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	dio24_query
*
*	Purpose:
*
*		Provide a visual wrapper for the DIO24_IOCTL_QUERY service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the query option to access.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int dio24_query(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs	= 0;
	const char*	ptr;
	s32			query	= set;
	int			ret;

	switch (query)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Query Error (#%ld)", (long) query);
			break;

		case DIO24_QUERY_CLOCK_COUNTER:	ptr	= "Clock Counter";		break;
		case DIO24_QUERY_COUNT:			ptr	= "Query Options";		break;
		case DIO24_QUERY_DEVICE_TYPE:	ptr	= "Device Type";		break;
		case DIO24_QUERY_DI:			ptr	= "Dedicated Input";	break;
		case DIO24_QUERY_IRQ_QTY:		ptr	= "IRQ Quantity";		break;
		case DIO24_QUERY_JUMPER_QTY:	ptr	= "Jumper Qty";			break;
		case DIO24_QUERY_JUMPER_SENSE:	ptr	= "Jumper Sense";		break;
		case DIO24_QUERY_JUMPER_VAL:	ptr	= "Jumper Value";		break;
	}

	if (verbose)
		gsc_label_index(ptr, index);

	ret		= dio24_ioctl(fd, DIO24_IOCTL_QUERY, &set);
	errs	+= ret ? 1 : 0;

	switch (query)
	{
		default:

			errs++;
			sprintf(buf, "%ld", (long) set);
			break;

		case DIO24_QUERY_COUNT:
		case DIO24_QUERY_JUMPER_QTY:
		case DIO24_QUERY_IRQ_QTY:

			sprintf(buf, "%ld", (long) set);
			break;

		case DIO24_QUERY_JUMPER_SENSE:

			sprintf(buf, "0x%lX", (long) set);
			break;

		case DIO24_QUERY_JUMPER_VAL:

			sprintf(buf, "0x%lX", (long) set);
			break;

		case DIO24_QUERY_CLOCK_COUNTER:
		case DIO24_QUERY_DI:

			strcpy(buf, set ? "Supported" : "Not Supported");
			break;

		case DIO24_QUERY_DEVICE_TYPE:

			if (set == GSC_DEV_TYPE_DIO24)
			{
				strcpy(buf, "DIO24");
			}
			else
			{
				errs++;
				sprintf(buf, "INVALID: %ld", (long) set);
			}

			break;
	}

	if (verbose)
	{
		if (errs)
			printf("FAIL <---  (%s)\n", buf);
		else
			printf("%s\n", buf);
	}

	if (get)
		get[0]	= set;

	return(errs);
}


