// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO24/DIO24_Linux_8.x.x.x_GSC_DN/utils/util_reg.c $
// $Rev: 52986 $
// $Date: 2023-05-03 16:42:23 -0500 (Wed, 03 May 2023) $

// DIO24: Utilities: source file

#include "main.h"



// macros *********************************************************************

#define	_GSC_REG(a)				"GSC " #a, DIO24_GSC_##a, 0, 0
#define	SHOW_FIELD(f,l,e,ls,d)	gsc_reg_field_show(width+10,24, value, (f),(l),(e),(ls),(d))



//*****************************************************************************
static int _frr_decode(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD(31, 24, 1, NULL, "Reserved"			);
	SHOW_FIELD(23, 16, 1, NULL, "Sub ID"			);
	SHOW_FIELD(15,  8, 1, NULL, "PCB Revision"		);
	SHOW_FIELD( 7,  0, 1, NULL, "Firmware Revision"	);
	return(0);
}



//*****************************************************************************
static int _bcr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	idle[]	= { "Idle",	"Active"	};

	SHOW_FIELD(31, 1, 1, NULL,	"Reserved"		);
	SHOW_FIELD( 0, 0, 1, idle,	"Board Reset"	);
	return(0);
}



//*****************************************************************************
static int _bsr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	state[]	= { "Disabled",	"Enabled"	};
	static const char*	off[]	= { "Off",	"On"	};

	s32	irq	= 0;

	dio24_query(fd, -1, 0, DIO24_QUERY_IRQ_QTY, &irq);

	if (irq)
	{
		SHOW_FIELD(31, 31, 1, state,	"Master Interrupt Enable"	);
		SHOW_FIELD(30, 18, 1, NULL,		"Reserved"					);
	}
	else
	{
		SHOW_FIELD(31, 18, 1, NULL,		"Reserved"					);
	}

	SHOW_FIELD(17, 17, 1, off,	"User Jumper 1"	);
	SHOW_FIELD(16, 16, 1, off,	"User Jumper 0"	);
	SHOW_FIELD(15,  0, 1, NULL,	"Reserved"		);
	return(0);
}



//*****************************************************************************
static int _iocr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	dir[]	= { "Input",	"Output"	};

	SHOW_FIELD(31, 10, 1, NULL,	"Reserved"				);
	SHOW_FIELD( 9,  9, 1, dir,	"Port C, Pin 7"			);
	SHOW_FIELD( 8,  8, 1, dir,	"Port C, Pin 6"			);
	SHOW_FIELD( 7,  7, 1, dir,	"Port C, Pin 5"			);
	SHOW_FIELD( 6,  6, 1, dir,	"Port C, Pin 4"			);
	SHOW_FIELD( 5,  5, 1, dir,	"Port C, Pin 3"			);
	SHOW_FIELD( 4,  4, 1, dir,	"Port C, Pin 2"			);
	SHOW_FIELD( 3,  3, 1, dir,	"Port C, Pin 1"			);
	SHOW_FIELD( 2,  2, 1, dir,	"Port C, Pin 0"			);
	SHOW_FIELD( 1,  1, 1, dir,	"Port B, All 8 Pins"	);
	SHOW_FIELD( 0,  0, 1, dir,	"Port A, All 8 Pins"	);
	return(0);
}



//*****************************************************************************
static int _ddor_decode(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD(31, 24, 1, NULL,	"Reserved"	);
	SHOW_FIELD(23, 16, 1, NULL,	"Port C"	);
	SHOW_FIELD(15,  8, 1, NULL,	"Port B"	);
	SHOW_FIELD( 7,  0, 1, NULL,	"Port A"	);
	return(0);
}



//*****************************************************************************
static int _ddir_decode(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD(31, 25, 1, NULL,	"Reserved"			);
	SHOW_FIELD(24, 24, 1, NULL,	"Dedicated Input"	);
	SHOW_FIELD(23, 16, 1, NULL,	"Port C"			);
	SHOW_FIELD(15,  8, 1, NULL,	"Port B"			);
	SHOW_FIELD( 7,  0, 1, NULL,	"Port A"			);
	return(0);
}



//*****************************************************************************
static int _isr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	status[]	= { "Negated",	"Asserted"	};

	s32	irq	= 0;

	dio24_query(fd, -1, 0, DIO24_QUERY_IRQ_QTY, &irq);

	if (supported)
	{
		supported	= irq;
	}
	else
	{
		SHOW_FIELD(31, 24, 1, NULL,		"Reserved"	);
		SHOW_FIELD(23, 23, 1, status,	"D23"		);
		SHOW_FIELD(22, 22, 1, status,	"D22"		);
		SHOW_FIELD(21, 21, 1, status,	"D21"		);
		SHOW_FIELD(20, 20, 1, status,	"D20"		);
		SHOW_FIELD(19, 19, 1, status,	"D19"		);
		SHOW_FIELD(18, 18, 1, status,	"D18"		);
		SHOW_FIELD(17, 17, 1, status,	"D17"		);
		SHOW_FIELD(16, 16, 1, status,	"D16"		);
		SHOW_FIELD(15, 15, 1, status,	"D15"		);
		SHOW_FIELD(14, 14, 1, status,	"D14"		);
		SHOW_FIELD(13, 13, 1, status,	"D13"		);
		SHOW_FIELD(12, 12, 1, status,	"D12"		);
		SHOW_FIELD(11, 11, 1, status,	"D11"		);
		SHOW_FIELD(10, 10, 1, status,	"D10"		);
		SHOW_FIELD( 9,  9, 1, status,	"D9"		);
		SHOW_FIELD( 8,  8, 1, status,	"D8"		);
		SHOW_FIELD( 7,  7, 1, status,	"D7"		);
		SHOW_FIELD( 6,  6, 1, status,	"D6"		);
		SHOW_FIELD( 5,  5, 1, status,	"D5"		);
		SHOW_FIELD( 4,  4, 1, status,	"D4"		);
		SHOW_FIELD( 3,  3, 1, status,	"D3"		);
		SHOW_FIELD( 2,  2, 1, status,	"D2"		);
		SHOW_FIELD( 1,  1, 1, status,	"D1"		);
		SHOW_FIELD( 0,  0, 1, status,	"D0"		);
	}

	return(supported);
}



//*****************************************************************************
static int _icr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	state[]	= { "Disabled",	"Enabled"	};

	s32	irq	= 0;

	dio24_query(fd, -1, 0, DIO24_QUERY_IRQ_QTY, &irq);

	if (supported)
	{
		supported	= irq;
	}
	else
	{
		SHOW_FIELD(31, 24, 1, NULL,		"Reserved"	);
		SHOW_FIELD(23, 23, 1, state,	"D23"		);
		SHOW_FIELD(22, 22, 1, state,	"D22"		);
		SHOW_FIELD(21, 21, 1, state,	"D21"		);
		SHOW_FIELD(20, 20, 1, state,	"D20"		);
		SHOW_FIELD(19, 19, 1, state,	"D19"		);
		SHOW_FIELD(18, 18, 1, state,	"D18"		);
		SHOW_FIELD(17, 17, 1, state,	"D17"		);
		SHOW_FIELD(16, 16, 1, state,	"D16"		);
		SHOW_FIELD(15, 15, 1, state,	"D15"		);
		SHOW_FIELD(14, 14, 1, state,	"D14"		);
		SHOW_FIELD(13, 13, 1, state,	"D13"		);
		SHOW_FIELD(12, 12, 1, state,	"D12"		);
		SHOW_FIELD(11, 11, 1, state,	"D11"		);
		SHOW_FIELD(10, 10, 1, state,	"D10"		);
		SHOW_FIELD( 9,  9, 1, state,	"D9"		);
		SHOW_FIELD( 8,  8, 1, state,	"D8"		);
		SHOW_FIELD( 7,  7, 1, state,	"D7"		);
		SHOW_FIELD( 6,  6, 1, state,	"D6"		);
		SHOW_FIELD( 5,  5, 1, state,	"D5"		);
		SHOW_FIELD( 4,  4, 1, state,	"D4"		);
		SHOW_FIELD( 3,  3, 1, state,	"D3"		);
		SHOW_FIELD( 2,  2, 1, state,	"D2"		);
		SHOW_FIELD( 1,  1, 1, state,	"D1"		);
		SHOW_FIELD( 0,  0, 1, state,	"D0"		);
	}

	return(supported);
}



// variables ******************************************************************

static gsc_reg_def_t	_gsc[]	=
{
	{ _GSC_REG(FRR),	0,	_frr_decode,	"Firmware Revision Register"					},
	{ _GSC_REG(BCR),	0,	_bcr_decode,	"Board Control Register"						},
	{ _GSC_REG(BSR),	0,	_bsr_decode,	"Board Status Register"							},
	{ _GSC_REG(IRSR),	1,	_icr_decode,	"Interrupt Rise Status Register"				},
	{ _GSC_REG(IFSR),	1,	_isr_decode,	"Interrupt Fall Status Register"				},
	{ _GSC_REG(IRCR),	1,	_icr_decode,	"Interrupt Rise Control Register"				},
	{ _GSC_REG(IFCR),	1,	_isr_decode,	"Interrupt Fall Control Register"				},
	{ _GSC_REG(DDIILR),	0,	_ddir_decode,	"Discrete Data Input Interrupt Latch Register"	},
	{ _GSC_REG(IOCR),	0,	_iocr_decode,	"I/O Control Register"							},
	{ _GSC_REG(DDOR),	0,	_ddor_decode,	"Discrete Data Output Register"					},
	{ _GSC_REG(DDIR),	0,	_ddir_decode,	"Discrete Data Input Register"					},

	{ NULL, 0, 0, 0, 0, NULL	}
};



//*****************************************************************************
static const gsc_reg_def_t* _find_reg(u32 reg, const gsc_reg_def_t* list)
{
	const gsc_reg_def_t*	def	= NULL;
	int						i;

	for (i = 0; list[i].name; i++)
	{
		if (reg == list[i].reg)
		{
			def	= &list[i];
			break;
		}
	}

	return(def);
}



/******************************************************************************
*
*	Function:	dio24_reg_get_def_id
*
*	Purpose:
*
*		Retrieve the register definition structure given the register id.
*
*	Arguments:
*
*		reg		The id of the register to access.
*
*	Returned:
*
*		NULL	The register id wasn't found.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* dio24_reg_get_def_id(u32 reg)
{
	const gsc_reg_def_t*	def;

	def	= _find_reg(reg, _gsc);
	return(def);
}



/******************************************************************************
*
*	Function:	dio24_reg_get_def_index
*
*	Purpose:
*
*		Retrieve the register definition structure based on an index.
*
*	Arguments:
*
*		index	The index of the register to access.
*
*	Returned:
*
*		NULL	The index doesn't correspond to a known register.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* dio24_reg_get_def_index(int index)
{
	const gsc_reg_def_t*	def;

	if (index < 0)
		def	= NULL;
	else if (index >= (SIZEOF_ARRAY(_gsc) - 1))
		def	= NULL;
	else
		def	= &_gsc[index];

	return(def);
}



/******************************************************************************
*
*	Function:	dio24_reg_get_desc
*
*	Purpose:
*
*		Retrieve the description of the specified register.
*
*	Arguments:
*
*		reg		The register whose description is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* dio24_reg_get_desc(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				desc;

	def	= _find_reg(reg, _gsc);

	if (def)
		desc	= def->desc;
	else
		desc	= "UNKNOWN";

	return(desc);
}



/******************************************************************************
*
*	Function:	dio24_reg_get_name
*
*	Purpose:
*
*		Retrieve the name of the specified register.
*
*	Arguments:
*
*		reg		The register whose name is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* dio24_reg_get_name(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				name;

	def	= _find_reg(reg, _gsc);

	if (def)
		name	= def->name;
	else
		name	= "UNKNOWN";

	return(name);
}



/******************************************************************************
*
*	Function:	dio24_reg_list
*
*	Purpose:
*
*		List the GSC registers and their values.
*
*	Arguments:
*
*		fd		The handle to access the device.
*
*		detail	List the register details?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int dio24_reg_list(int fd, int detail)
{
	int	errs;

	errs	= gsc_reg_list(fd, _gsc, detail, dio24_reg_read);
	return(errs);
}



